@@ -1,25 +1,22 @@
 #!/usr/bin/env python3
 
-from http import HTTPStatus
-
 import click
 import click_log
 import glob
 import json
 import logging
-import requests
 from pathlib import Path
 from pprint import pprint
 from tempfile import TemporaryFile, NamedTemporaryFile
-from typing import List, Dict
+from typing import List, Dict, Union
 from zipfile import ZipFile, ZIP_DEFLATED
 
-from requests.sessions import Session
-
 from fluidtopics.connector.model.metadata import SemanticMetadata
 
 from .ftmap import FTMap
 from .metadata import FT_ORIGIN_ID, FT_LANG, get_md_metas
+from .errors import MD2FTError
+from .ftsession import FTSession
 from . import __version__ as md2ft_version
 
 logger = logging.getLogger("fluidtopics.markdown")
@@ -28,22 +25,6 @@ click_log.basic_config(logger)
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 
-class MD2FTError(Exception):
-    pass
-
-
-def prepare_ft_session(portal_base_url: str, user: str, passwd: str) -> Session:
-    "Prepare an FT session to call API"
-    ft_session = requests.Session()
-    login_url = portal_base_url + "/api/authentication/login"
-    rep = ft_session.post(login_url, json={"login": user, "password": passwd})
-    if not rep.ok:
-        raise MD2FTError(
-            f"Cannot log in to FT portal {login_url} (user={user}, password=XXX): {rep.content}"
-        )
-    return ft_session
-
-
 def prune_empty_dir(content: Dict[str, dict]) -> Dict[str, dict]:
     """Remove empty dir (i.e. content node with child) which doesn't have any non-empty child"""
     # end of recursion 1: node is empty
@@ -317,47 +298,28 @@ def do_collect(
 
 def do_publish(
     zip: Path,
-    portal_base_url: str,
-    user: str,
-    passwd: str,
+    ft_session: FTSession,
     source_id: str,
     dry_run: bool,
 ):
     """Upload and publish an FTML archive to an FT portal."""
     if not dry_run:
-        ft_session = prepare_ft_session(portal_base_url, user, passwd)
         # Verify that the server is reachable and has FTML source exists
-        list_sources_url = f"{portal_base_url}/api/admin/khub/sources"
-        rep = ft_session.get(list_sources_url)
-        if rep.status_code in [HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN]:
-            raise MD2FTError(
-                f"Server rejected credentials as Unauthorized ({rep.status_code})."
-            )
-        elif rep.status_code == HTTPStatus.NOT_FOUND:
-            raise MD2FTError(
-                f'The instance id or the API in "{list_sources_url}" is not found ({rep.status_code}).'
-            )
-        elif not rep.ok:
-            raise MD2FTError(rep.text)
+        ft_session.get("/api/admin/khub/sources", json_result=False)
 
     # Upload FTML zip archive
     if type(zip) == str:
         zipfile = {"file": ("md2ft_" + Path(zip).name, Path(zip).open("rb"))}
     else:
         zipfile = {"file": ("md2ft_" + Path(zip.name).name, zip)}
-    post_url = f"{portal_base_url}/api/admin/khub/sources/{source_id}/upload"
 
     if not dry_run:
-        rep = ft_session.post(post_url, files=zipfile)
-        if rep.status_code == HTTPStatus.OK:
-            answer = rep.json()
-            logger.info(f"{zipfile['file'][0]} file published on {portal_base_url}.")
-            pprint(answer)
-        else:
-            raise MD2FTError(rep.text)
+        answer = ft_session.post(f"/api/admin/khub/sources/{source_id}/upload", files=zipfile)
+        logger.info(f"{zipfile['file'][0]} file published on {ft_session.portal_base_url}.")
+        pprint(answer)
     else:
         logger.info(
-            f"DRY RUN: {zipfile['file'][0]} file NOT published on {portal_base_url}."
+            f"DRY RUN: {zipfile['file'][0]} file NOT published."
         )
 
 
@@ -377,9 +339,7 @@ def _get_meta_values(key: str, metadatas: List[dict], first_value_only: bool):
 
 
 def do_unpublish(
-    portal_base_url: str,
-    user: str,
-    passwd: str,
+    ft_session: FTSession,
     source_id: str,
     dry_run: bool,
     do_it: bool,
@@ -391,29 +351,14 @@ def do_unpublish(
     - title : delete the publication with the corresponding title
     """
     if not dry_run:
-        # Prepare a FT session
-        ft_session = prepare_ft_session(portal_base_url, user, passwd)
 
         # Get all the maps from md2ft source
-        list_maps = (
-            f"{portal_base_url}/api/admin/khub/publications?ft:sourceId={source_id}"
-        )
-        r = ft_session.get(list_maps)
-        if r.status_code == HTTPStatus.UNAUTHORIZED:
-            logger.fatal(
-                f"Server rejected credentials as Unauthorized ({r.status_code})."
-            )
-        if r.status_code == HTTPStatus.NOT_FOUND:
-            logger.fatal(
-                f'The instance id or the API in "{list_maps}" is not found ({r.status_code}).'
-            )
-        r.raise_for_status()
-        list_maps_json = r.json()
+        list_maps_json = ft_session.get(f"/api/admin/khub/publications?ft:sourceId={source_id}")
 
-        # Exist if no publication or source does not exist
+        # Exit if no publication or source does not exist
         if list_maps_json["count"] == 0:
             logger.info(
-                f"No publications in this source_id or the source: {source_id} doesn't exist"
+                f"No publications in this source_id or the <{source_id}> source doesn't exist"
             )
             return
 
@@ -457,21 +402,16 @@ def do_unpublish(
         if selected_documents:
             # Iter on each publication to delete
             for id_to_delete, metas in selected_documents.items():
-                delete_map = f"{portal_base_url}/api/admin/khub/maps/{id_to_delete}"
                 # Delete the publication if enable
                 if do_it:
-                    r = ft_session.delete(delete_map)
-
-                    if r.status_code == HTTPStatus.OK:
-                        answer = r.json()
-                        logger.info(
-                            f" ID MAP {id_to_delete} deleted on {portal_base_url}."
-                        )
-                        pprint(answer)
-                    r.raise_for_status()
+                    answer = ft_session.delete(f"/api/admin/khub/maps/{id_to_delete}")
+                    logger.info(
+                        f" ID MAP {id_to_delete} deleted on {ft_session.portal_base_url}."
+                    )
+                    pprint(answer)
                 else:
                     logger.info(
-                        f"Would delete document id={id_to_delete} (whose metas={metas}) on {portal_base_url}."
+                        f"Would delete document id={id_to_delete} (whose metas={metas}) on {ft_session.portal_base_url}."
                     )
             if not do_it:
                 logger.info(
@@ -479,11 +419,11 @@ def do_unpublish(
                 )
         else:
             logger.info(
-                f"No publications selected with <<{select}>> from source_id : {source_id} on portal {portal_base_url}"
+                f"No publications selected with <<{select}>> from source_id : {source_id} on portal {ft_session.portal_base_url}"
             )
     else:
         logger.info(
-            f"DRY RUN: No publications deleted from source_id : {source_id} on portal : {portal_base_url}."
+            f"DRY RUN: No publications deleted from source_id : {source_id} on portal."
         )
 
 
@@ -533,24 +473,32 @@ publish_opts = [
         required=True,
         type=click.STRING,
     ),
+    click.option(
+        "-a",
+        "--api-key",
+        type=click.STRING,
+        metavar="<apikey>",
+        help="The FT API Key",
+        envvar='MD2FT_API_KEY',
+        show_envvar=True,
+    ),
     click.option(
         "-u",
         "--user",
-        required=True,
         type=click.STRING,
         metavar="<user>",
         help="The FT user",
-        prompt="Please, provide a FT user account",
+        envvar='MD2FT_USER',
+        show_envvar=True,
     ),
     click.option(
         "-p",
         "--passwd",
-        required=True,
         type=click.STRING,
         metavar="<password>",
         help="The FT password",
-        prompt="Please, provide the FT user password",
-        hide_input=True,
+        envvar=['MD2FT_PASSWD', 'MD2FT_PASSWORD'],
+        show_envvar=True,
     ),
     click.option(
         "-s",
@@ -571,6 +519,22 @@ publish_opts = [
 ]
 
 
+def verify_credentials(portal_base_url: str, creds: dict, dry_run: bool) -> Union[FTSession, None]:
+    if dry_run:
+        logger.info(f"DRY RUN authentication on {portal_base_url}")
+        return None
+    ft_session = FTSession(portal_base_url)
+    if creds.get("api_key") is not None:
+        logger.info("Using API Key credentials")
+        ft_session.set_api_key(creds.get("api_key"))
+        return ft_session
+    if creds.get("user") is not None and creds.get("password") is not None:
+        logger.info("Using user/passwd credentials")
+        ft_session.login(creds.get("user"), creds.get("password"))
+        return ft_session
+    raise click.UsageError("You must specify either an API Key (--api-key) or user **and** passwd (--user/--passwd)")
+
+
 def add_options(options):
     def _add_options(func):
         for option in reversed(options):
@@ -618,18 +582,20 @@ def collect_and_publish(
     implicit_meta: bool,
     dev_mode: bool,
     portal_base_url: str,
+    api_key: str,
     user: str,
     passwd: str,
     source_id: str,
     dry_run: bool,
 ):
     """Collect upload and publish documentation to an FT portal."""
+    session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
     with NamedTemporaryFile(mode="r+b", suffix=".zip") as tempzip:
         logger.debug(f"tempzip={tempzip.name}")
         do_collect(input, tempzip, included_audiences, implicit_meta)
         # rewind the file for next read
         tempzip.seek(0)
-        do_publish(tempzip, portal_base_url, user, passwd, source_id, dry_run)
+        do_publish(tempzip, session, source_id, dry_run)
 
 
 @cli.command("publish")
@@ -642,13 +608,15 @@ def collect_and_publish(
 def publish(
     zip: Path,
     portal_base_url: str,
+    api_key: str,
     user: str,
     passwd: str,
     source_id: str,
     dry_run: bool,
 ):
     """Upload and publish an FTML archive to an FT portal."""
-    do_publish(zip, portal_base_url, user, passwd, source_id, dry_run)
+    session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
+    do_publish(zip, session, source_id, dry_run)
 
 
 @cli.command("unpublish")
@@ -673,6 +641,7 @@ def publish(
 )
 def delete_book(
     portal_base_url: str,
+    api_key: str,
     user: str,
     passwd: str,
     source_id: str,
@@ -680,8 +649,12 @@ def delete_book(
     do_it: bool,
     select: str,
 ):
-    """Delete all publications older than the latest ones from an FT portal (from last md2ft publish)"""
-    do_unpublish(portal_base_url, user, passwd, source_id, dry_run, do_it, select)
+    """Delete selected publications from an FT portal.
+
+    The default selection is using ft:lastEdition metadata.
+    """
+    session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
+    do_unpublish(session, source_id, dry_run, do_it, select)
 
 
 if __name__ == "__main__":
